@@ -18,21 +18,22 @@
 #
 #  SYNOPSIS
 #
-#     fmr(y, dist="normal", pmu=NULL, pmix=NULL, pshape=NULL, mu=NULL,
-#	mix=NULL, linear=NULL, censor="right",exact=F, wt=1, delta=1,
-#	print.level=0, typsiz=abs(p), ndigit=10, gradtol=0.00001,
-#	stepmax=10*sqrt(p%*%p), steptol=0.00001, iterlim=100, fscale=1)
+#     fmr(y, distribution="normal", mu=NULL, mix=NULL, linear=NULL,
+#	pmu=NULL, pmix=NULL, pshape=NULL, censor="right", exact=F,
+#	wt=1, delta=1, envir=sys.frame(sys.parent()), print.level=0,
+#	typsiz=abs(p), ndigit=10, gradtol=0.00001, stepmax=10*sqrt(p%*%p),
+#	steptol=0.00001, iterlim=100, fscale=1)
 #
 #  DESCRIPTION
 #
 #    A function to fit nonlinear regression models with a variety of
 # distributions and a mixture in the tail(s).
 
-fmr <- function(y, dist="normal", pmu=NULL, pmix=NULL, pshape=NULL, mu=NULL,
-	mix=NULL, linear=NULL, censor="right",exact=F, wt=1, delta=1,
-	print.level=0, typsiz=abs(p), ndigit=10, gradtol=0.00001,
-	stepmax=10*sqrt(p%*%p), steptol=0.00001, iterlim=100,
-	fscale=1){
+fmr <- function(y, distribution="normal", mu=NULL, mix=NULL, linear=NULL,
+	pmu=NULL, pmix=NULL, pshape=NULL, censor="right", exact=F,
+	wt=1, delta=1, envir=sys.frame(sys.parent()), print.level=0,
+	typsiz=abs(p), ndigit=10, gradtol=0.00001, stepmax=10*sqrt(p%*%p),
+	steptol=0.00001, iterlim=100, fscale=1){
 
 pinvgauss <- function(y,m,s){
 	t <- y/m
@@ -52,43 +53,60 @@ plevy <- function(y, m, s)
 		pts=as.integer(5),
 		max=as.integer(16),
 		err=integer(1),
-		res=double(n))$res
+		res=double(n),
+		DUP=F)$res
 
 call <- sys.call()
-if(!missing(dist)&&!is.function(dist)){
-	dist <- match.arg(dist,c("binomial","beta binomial","double binomial",
-	"mult binomial","Poisson","negative binomial","double Poisson",
-	"mult Poisson","gamma count","Consul","geometric","normal",
-	"inverse Gauss","logistic","exponential","gamma","Weibull",
+if(!missing(distribution)&&!is.function(distribution)){
+	distribution <- match.arg(distribution,c("binomial","beta binomial",
+	"double binomial","mult binomial","Poisson","negative binomial",
+	"double Poisson","mult Poisson","gamma count","Consul","geometric",
+	"normal","inverse Gauss","logistic","exponential","gamma","Weibull",
 	"extreme value","Pareto","Cauchy","Student t","Laplace","Levy"))}
 if(!missing(pmu))npl <- length(pmu)
 else npl <- 0
 if(!missing(pmix))npm <- length(pmix)
 else npm <- 0
-sht <- dist!="binomial"&&dist!="Poisson"&&dist!="exponential"&&dist!="geometric"
+sht <- distribution!="binomial"&&distribution!="Poisson"&&
+	distribution!="exponential"&&distribution!="geometric"
 if(sht&&missing(pshape))
 	stop("An estimate of the shape parameter must be given")
 np <- npl+npm+sht
-p <- c(pmu,pmix,pshape)
-if(is.function(dist)){
-	fcn <- dist
-	dist <- "own"}
-if(inherits(y,"response")){
-	if(is.null(y$censor))y <- y$y
-	else y <- cbind(y$y,y$censor)}
-if(dist=="Poisson"||dist=="negative binomial"||dist=="double Poisson"||dist=="mult Poisson"||dist=="gamma count"||dist=="Consul"){
-	if(!is.vector(y,mode="double"))stop("y must be a vector")
+if(is.function(distribution)){
+	fcn <- distribution
+	distribution <- "own"}
+if(inherits(y,"repeated")){
+	if(inherits(envir,"repeated")&&(length(y$response$nobs)!=length(envir$response$nobs)||any(y$response$nobs!=envir$response$nobs)))stop("y and envir objects are incompatible")
+	if(!is.null(y$response$wt)&&!is.na(y$response$wt))wt <- y$response$wt
+	if(!is.null(y$response$delta))delta <- y$response$delta
+	if(!is.null(y$response$censor)&&any(y$response$censor!=1))y <- cbind(y$response$y,y$response$censor)
+	else if(!is.null(y$response$n))y <- cbind(y$response$y,y$response$n-y$response$y)
+	else y <- y$response$y}
+else if(inherits(y,"response")){
+	if(!is.null(y$wt)&&!is.na(y$wt))wt <- y$wt
+	if(!is.null(y$delta))delta <- y$delta
+	if(!is.null(y$censor)&&any(y$censor!=1))y <- cbind(y$y,y$censor)
+	else if(!is.null(y$n))y <- cbind(y$y,y$n-y$y)
+	else y <- y$y}
+if(distribution=="Poisson"||distribution=="negative binomial"||
+	distribution=="double Poisson"||distribution=="mult Poisson"||
+	distribution=="gamma count"||distribution=="Consul"){
+	if(!is.vector(y,mode="numeric"))stop("y must be a vector")
 	n <- length(y)
 	censor <- NULL
 	cens <- ifelse(y==0,1,0)}
 else {
 	if(length(dim(y))!=2||ncol(y)!=2)
 		stop(paste("Two column matrix required for response:",
-		if(dist=="binomial"||dist=="beta binomial"||dist=="double binomial"||dist=="mult binomial")"successes and failures"
+		if(distribution=="binomial"||distribution=="beta binomial"||
+			distribution=="double binomial"||
+			distribution=="mult binomial")"successes and failures"
 		else "times and censor indicator"))
 	else {
 		n <- nrow(y)
-		if(dist=="binomial"||dist=="beta binomial"||dist=="double binomial"||dist=="mult binomial"){
+		if(distribution=="binomial"||distribution=="beta binomial"||
+			distribution=="double binomial"||
+			distribution=="mult binomial"){
 			if(missing(censor))
 				stop("Censoring must be left, right, or both")
 			if(censor!="left"&&censor!="right"&&censor!="both")
@@ -125,103 +143,169 @@ else {
 			l <- as.integer(censor=="right"&y[,2]==-1)
 			lc <- ifelse(censor=="left",1,0)
 			rc <- ifelse(censor=="right",-1,1)}}
-	if(dist=="double Poisson"||dist=="mult Poisson")
+	if(distribution=="double Poisson"||distribution=="mult Poisson")
 				my <- min(3*max(y),100)}
-if((dist!="normal"&&dist!="logistic"&&dist!="Cauchy"&&
-	dist!="Laplace"&&dist!="Student t"&&dist!="Poisson"&&
-	dist!="negative binomial"&&dist!="Consul"&&dist!="double Poisson"&&
-	dist!="mult Poisson"&&dist!="gamma count"&&dist!="binomial"&&
-	dist!="beta binomial"&&dist!="double binomial"&&
-	dist!="mult binomial")&&(any(y[,1]<=0)))
-	stop("All response values must be > 0")
-else if((dist=="Poisson"||dist=="negative binomial"||dist=="gamma count"||
-		dist=="double Poisson"||dist=="mult Poisson"||dist=="Consul"||
-		dist=="binomial"||dist=="beta binomial"||
-		dist=="double binomial"||dist=="mult binomial")
-		&&(any(y<0)))stop("All response values must be >= 0")
+if((distribution!="normal"&&distribution!="logistic"&&distribution!="Cauchy"&&
+	distribution!="Laplace"&&distribution!="Student t"&&
+	distribution!="Poisson"&&distribution!="negative binomial"&&
+	distribution!="Consul"&&distribution!="double Poisson"&&
+	distribution!="mult Poisson"&&distribution!="gamma count"&&
+	distribution!="binomial"&& distribution!="beta binomial"&&
+	distribution!="double binomial"&&distribution!="mult binomial")&&
+	(any(y[,1]<=0)))stop("All response values must be > 0")
+else if((distribution=="Poisson"||distribution=="negative binomial"||
+	distribution=="gamma count"||distribution=="double Poisson"||
+	distribution=="mult Poisson"||distribution=="Consul"||
+	distribution=="binomial"||distribution=="beta binomial"||
+	distribution=="double binomial"||distribution=="mult binomial")
+	&&(any(y<0)))stop("All response values must be >= 0")
 if(min(wt)<0)stop("All weights must be non-negative")
 if(length(wt)==1)wt <- rep(wt,n)
 if(length(delta)==1)delta <- rep(delta,n)
 lin1 <- lin2 <- NULL
 if(is.list(linear)){
 	lin1 <- linear[[1]]
-	lin2 <- linear[[2]]
-	lmix <- is.language(lin2)}
-else {
-	lin1 <- linear
-	lmix <- FALSE}
-if(is.language(mu))lin1 <- mu
-if(is.language(mix)){
-	lin2 <- mix
-	lmix <- is.language(lin2)}
-nlp <- npl
-if(is.language(lin1)){
-	mt <- terms(lin1)
-	if(is.numeric(mt[[2]])){
-		dm1 <- matrix(1)
-		colnames(dm1) <- "(Intercept)"
-		npt1 <- 1
-		if(!is.function(mu)){
-			mu1 <- function(p) p[1]*rep(1,n)
-			nlp <- 1}
-		else mu1 <- function (p) mu(p, p[1]*rep(1,n))}
+	lin2 <- linear[[2]]}
+else lin1 <- linear
+if(inherits(mu,"formula"))lin1 <- mu
+if(inherits(mix,"formula"))lin2 <- mix
+lin1a <- lin2a <- mu2 <- mixt2 <- name <- NULL
+if(inherits(envir,"repeated")||inherits(envir,"tccov")){
+	type <- if(inherits(envir,"repeated"))"repeated"
+		else "tccov"
+	if(inherits(lin1,"formula")){
+		lin1a <- finterp(lin1)
+		class(lin1) <- c(class(lin1),type)}
+	if(inherits(lin2,"formula")){
+		lin2a <- finterp(lin2)
+		class(lin2) <- c(class(lin2),type)}
+	name <- paste(deparse(substitute(envir)))
+	if(is.function(mu)){
+		mu2 <- mu
+		attributes(mu2) <- attributes(fnenvir(mu))
+		class(mu) <- type
+		mu <- fnenvir(mu,envir=envir,name=name)}
+	if(is.function(mix)){
+		mixt2 <- mix
+		attributes(mixt2) <- attributes(fnenvir(mix))
+		class(mix) <- type
+		mix <- fnenvir(mix,envir=envir,name=name)}}
+if(inherits(lin1,"formula")){
+	mu1 <- finterp(lin1,envir=envir,name=name)
+	npt1 <- length(attr(mu1,"parameters"))
+	if(is.matrix(attr(mu1,"model"))){
+		if(all(dim(attr(mu1,"model"))==1)){
+			if(is.function(mu)){
+				lin1 <- mu1
+				mu1 <- function(p) mu(p,p[1]*rep(1,n))}
+			else {
+				tmp <- attributes(mu1)
+				mu1 <- function(p) p[1]*rep(1,n)
+				attributes(mu1) <- tmp}}
+		else {
+			if(nrow(attr(mu1,"model"))!=n)stop("mu model matrix does not match number of response observations")
+			if(is.function(mu)){
+				dm1 <- attr(mu1,"model")
+				lin1 <- mu1
+				mu1 <- function(p) mu(p,dm1%*%p[1:npt1])}}}
 	else {
-		mf <- model.frame(mt,sys.frame(sys.parent()),na.action=na.fail)
-		dm1 <- model.matrix(mt, mf)
-		npt1 <- ncol(dm1)
-		if(!is.function(mu)){
-			mu1 <- function(p) dm1 %*% p[1:npt1]
-			nlp <- npt1}
-		else mu1 <- function (p) mu(p, dm1 %*% p[1:npt1])}
+		if(npl!=npt1){
+			cat("\nParameters are ")
+			cat(attr(mu1,"parameters"),"\n")
+			stop(paste("pmu should have",npt1,"estimates"))}
+		if(is.list(pmu)){
+			if(!is.null(names(pmu))){
+				o <- match(attr(mu1,"parameters"),names(pmu))
+				pmu <- unlist(pmu)[o]
+				if(sum(!is.na(o))!=length(pmu))stop("invalid estimates for mu - probably wrong names")}
+			else pmu <- unlist(pmu)}}
 	if(npl<npt1)stop("Not enough initial estimates for mu")}
 else if(!is.function(mu)){
 	mu1 <- function(p) p[1]*rep(1,n)
-	nlp <- 1}
-else mu1 <- mu
-if(nlp!=npl)
-	stop("Number of initial estimates for mu does not correspond to model")
+	npt1 <- 1}
+else {
+	mu1 <- mu
+	if(length(mu1(pmu))==1)mu1 <- function(p) mu(p)*rep(1,n)}
+if(is.null(attributes(mu1))){
+	attributes(mu1) <- if(is.function(mu)){
+		if(!inherits(mu,"formulafn"))attributes(fnenvir(mu))
+		else attributes(mu)}
+		else attributes(fnenvir(mu1))}
+nlp <- if(is.function(mu)){
+		if(is.null(lin1))length(attr(mu1,"parameters"))
+		else length(attr(mu1,"parameters"))-1+npt1}
+       else npt1
+if(nlp!=npl)stop(paste("pmu should have",nlp,"initial estimates"))
+if(any(is.na(mu1(pmu))))stop("The location model returns NAs: probably invalid initial values")
 npl1 <- npl+1
-nlp <- npm
-if(lmix){
-	mt <- terms(lin2)
-	if(is.numeric(mt[[2]])){
-		dm2 <- matrix(1)
-		colnames(dm2) <- "(Intercept)"
-		npt2 <- 1
-		if(!is.function(mix)) mixt <- function(p) {
-			mf <- p[npl1]*rep(1,n)
-			exp(mf)/(1+exp(mf))}
-		else mixt <- function(p) {
-			mf <- mix(p[npl1:np], p[npl1]*rep(1,n))
-			exp(mf)/(1+exp(mf))}}
-	else {
-		mf <- model.frame(mt,sys.frame(sys.parent()),na.action=na.fail)
-		dm2 <- model.matrix(mt, mf)
-		npt2 <- ncol(dm2)
-		if(!is.function(mix)){
-			mixt <- function(p) {
-				mf <- dm2 %*% p[npl1:(npl1+npt2-1)]
+if(inherits(lin2,"formula")){
+	mixt1 <- finterp(lin2,envir=envir,start=npl1,name=name)
+	npt2 <- length(attr(mixt1,"parameters"))
+	if(is.matrix(attr(mixt1,"model"))){
+		if(all(dim(attr(mixt1,"model"))==1)){
+			if(is.function(mix)){
+				lin2 <- mixt1
+				mixt <- function(p) {
+					mf <- mix(p[npl1:np],p[npl1]*rep(1,n))
+					exp(mf)/(1+exp(mf))}}
+			else {
+				mixt <- function(p) {
+					mf <- p[npl1]*rep(1,n)
+					exp(mf)/(1+exp(mf))}
+				mixt2 <- fnenvir(function(p) {
+					mf <- p[1]*rep(1,n)
+					exp(mf)/(1+exp(mf))})
+				attributes(mixt) <- attributes(mixt1)}
+			rm(mixt1)}
+		else {
+			if(nrow(attr(mixt1,"model"))!=n)stop("mix model matrix does not match number of response observations")
+			dm2 <- attr(mixt1,"model")
+			if(is.function(mix))mixt <- function(p) {
+				lin2 <- mixt1
+				mf <- mix(p[npl1:np],dm2%*%p[npl1:(npl1+npt2-1)])
 				exp(mf)/(1+exp(mf))}
-			nlp <- npt2}
-		else mixt <- function (p) {
-			mf <- mix(p[npl1:np], dm2 %*% p[npl1:(nlp1+npt2-1)])
-			exp(mf)/(1+exp(mf))}}
-	if(npm<npt2)stop("Not enough initial estimates for mix")}
+			else {
+				mixt <- function(p) {
+					mf <- dm2%*%p[npl1:(npl1+npt2-1)]
+					exp(mf)/(1+exp(mf))}
+				attributes(mixt) <- attributes(mixt1)}}}
+	else {
+		if(npm!=npt2){
+			cat("\nParameters are ")
+			cat(attr(mixt1,"parameters"),"\n")
+			stop(paste("pmix should have",npt2,"estimates"))}
+		mixt <- function(p) {
+			mf <- mixt1(p)
+			exp(mf)/(1+exp(mf))}
+		attributes(mixt) <- attributes(mixt1)
+		if(is.list(pmix)){
+			if(!is.null(names(pmix))){
+				o <- match(attr(mixt,"parameters"),names(pmix))
+				pmix <- unlist(pmix)[o]
+				if(sum(!is.na(o))!=length(pmix))stop("invalid estimates for mix - probably wrong names")}
+			else pmix <- unlist(pmix)}}}
 else if(!is.function(mix)){
 	mixt <- function(p) exp(p[npl1])/(1+exp(p[npl1]))*rep(1,n)
-	nlp <- 1}
+	mixt2 <- fnenvir(function(p) exp(p[1])/(1+exp(p[1]))*rep(1,n))
+	npt2 <- 1}
 else mixt <- function(p) {
 	mf <- mix(p[npl1:np])
 	exp(mf)/(1+exp(mf))}
-if(nlp!=npm)
-	stop("Number of initial estimates for mix does not correspond to model")
-if(!is.numeric(mu1(pmu)))
-	stop("The location function must return numerical values")
-if(dist=="Levy"&&any(y[,1]<=mu1(p)))
+if(is.null(attributes(mixt))){
+	attributes(mixt) <- if(is.function(mix)){
+		if(!inherits(mix,"formulafn"))attributes(fnenvir(mix))
+		else attributes(mix)}
+		else attributes(fnenvir(mixt))}
+nlp <- if(is.function(mix)){
+		if(is.null(lin2))length(attr(mixt,"parameters"))
+		else length(attr(mixt,"parameters"))-1+npt2}
+       else npt2
+if(nlp!=npm)stop(paste("pshape should have",nlp,"initial estimates"))
+p <- c(pmu,pmix,pshape)
+if(distribution=="Levy"&&any(y[,1]<=mu1(p)))
 	stop("location parameter must be strictly less than corresponding observation")
-if(sht&&!is.numeric(mixt(p)))
-	stop("The mix function must return numerical values")
-ret <- switch(dist,
+if(sht&&any(is.na((mixt(p)))))stop("The mix function returns NAs: probably invalid initial values")
+ret <- switch(distribution,
 	binomial={
 		fcn <- function(p) {
 			m <- mu1(p)
@@ -244,7 +328,7 @@ ret <- switch(dist,
 				as.integer(y[,1]),as.integer(nn),
 				as.double(mu1(p)),as.double(exp(p[np])),
 				as.integer(n),as.double(wt),
-				res=double(n))$res))}
+				res=double(n),DUP=F)$res))}
 		const <- 0},
 	"mult binomial"={
 		fcn <- function(p) {
@@ -252,7 +336,7 @@ ret <- switch(dist,
 				as.integer(y[,1]),as.integer(nn),
 				as.double(mu1(p)),as.double(exp(p[np])),
 				as.integer(n),as.double(wt),
-				res=double(n))$res))}
+				res=double(n),DUP=F)$res))}
 		const <- 0},
 	Poisson={
 		fcn <- function(p) {
@@ -273,14 +357,14 @@ ret <- switch(dist,
 			-wt*log((1-s)*cens+s*exp(.C("ddp",as.integer(y),
 				as.integer(my),as.double(mu1(p)),
 				as.double(exp(p[np])),as.integer(length(y)),
-				as.double(wt),res=double(length(y)))$res))}
+				as.double(wt),res=double(length(y)),DUP=F)$res))}
 		const <- 0},
 	"mult Poisson"={
 		fcn <- function(p) {
 			-wt*log((1-s)*cens+s*exp(.C("dmp",as.integer(y),
 				as.integer(my),as.double(mu1(p)),
 				as.double(exp(p[np])),as.integer(length(y)),
-				as.double(wt),res=double(length(y)))$res))}
+				as.double(wt),res=double(length(y)),DUP=F)$res))}
 		const <- 0},
 	"gamma count"={
 		fcn <- function(p) {
@@ -305,8 +389,9 @@ ret <- switch(dist,
 				m <- mu1(p)
 				s <- mixt(p)
 				t <- exp(p[np]/2)
-				pn <- pnorm(y[,1]-delta/2,m,t)
-				-wt*log(s*cens*(pnorm(y[,1]+delta/2,m,t)-pn)
+				pn <- pnorm(y[,1],m,t)
+				-wt*log(s*cens*(pnorm(y[,1]+delta/2,m,t)-
+					pnorm(y[,1]-delta/2,m,t))
 					+(1-cens)*((1+s*(rc*pn-lc))*b
 					+s*(r+pn*(l-r))))}
 			const <- 0}
@@ -314,11 +399,10 @@ ret <- switch(dist,
 			fcn <- function(p) {
 				m <- mu1(p)
 				s <- mixt(p)
-				pn <- pnorm(y[,1]-delta/2,m,exp(p[np]/2))
+				pn <- pnorm(y[,1],m,exp(p[np]/2))
 				-wt*log(s*cens*exp((-(p[np]+(y[,1]-m)^2
-					*exp(-p[np]))/2))
-					+(1-cens)*((1+s*(rc*pn-lc))*b
-					+s*(r+pn*(l-r))))}
+					*exp(-p[np]))/2))+(1-cens)*
+					((1+s*(rc*pn-lc))*b+s*(r+pn*(l-r))))}
 			const <- wt*cens*(log(2*pi)/2-log(delta))}},
         "inverse Gauss"={
 		if(exact){
@@ -326,8 +410,9 @@ ret <- switch(dist,
 				m <- mu1(p)
 				s <- mixt(p)
 				t <- exp(p[np])
-				pit <- pinvgauss(y[,1]-delta/2,m,t)
-				-wt*log(s*cens*(pinvgauss(y[,1]+delta/2,m,t)-pit)
+				pit <- pinvgauss(y[,1],m,t)
+				-wt*log(s*cens*(pinvgauss(y[,1]+delta/2,m,t)-
+					pinvgauss(y[,1]-delta/2,m,t))
 					+(1-cens)*((1+s*(rc*pit-lc))*b
 					+s*(r+pit*(l-r))))}
 			const <- 0}
@@ -336,7 +421,7 @@ ret <- switch(dist,
 				m <- mu1(p)
 				s <- mixt(p)
 				t <- exp(p[np])
-				pit <- pinvgauss(y[,1]-delta/2,m,t)
+				pit <- pinvgauss(y[,1],m,t)
 				-wt*log(s*cens*exp(-(p[np]+(y[,1]-m)^2/
 					(y[,1]*t*m^2))/2)
 					+(1-cens)*((1+s*(rc*pit-lc))*b
@@ -348,8 +433,9 @@ ret <- switch(dist,
 				m <- mu1(p)
 				s <- mixt(p)
 				t <- exp(p[np])*sqrt(3)/pi
-				pl <- plogis(y[,1]-delta/2,m,t)
-				-wt*log(s*cens*(plogis(y[,1]+delta/2,m,t)-pl)
+				pl <- plogis(y[,1],m,t)
+				-wt*log(s*cens*(plogis(y[,1]+delta/2,m,t)-
+					plogis(y[,1]-delta/2,m,t))
 					+(1-cens)*((1+s*(rc*pl-lc))*b
 					+s*(r+pl*(l-r))))}
 			const <- 0}
@@ -359,7 +445,7 @@ ret <- switch(dist,
 				s <- mixt(p)
 				t <- exp(p[np])*sqrt(3)/pi
 				y1 <- (y[,1]-m)/t
-				pl <- plogis(y[,1]-delta/2,m,t)
+				pl <- plogis(y[,1],m,t)
 				-wt*log(s*cens*exp(-y1-log(t)
 					-2*log(1+exp(-y1)))
 					+(1-cens)*((1+s*(rc*pl-lc))*b
@@ -371,8 +457,9 @@ ret <- switch(dist,
 				m <- mu1(p)
 				s <- mixt(p)
 				t <- exp(p[np])
-				ps <- pt(y[,1]-delta/2-m,t)
-				-wt*log(s*cens*(pt(y[,1]+delta/2-m,t)-ps)
+				ps <- pt(y[,1]-m,t)
+				-wt*log(s*cens*(pt(y[,1]+delta/2-m,t)-
+					pt(y[,1]-delta/2-m,t))
 					+(1-cens)*((1+s*(rc*ps-lc))*b
 					+s*(r+ps*(l-r))))}
 			const <- 0}
@@ -381,7 +468,7 @@ ret <- switch(dist,
 				m <- mu1(p)
 				s <- mixt(p)
 				t <- exp(p[np])
-				ps <- pt(y[,1]-delta/2-m,t)
+				ps <- pt(y[,1]-m,t)
 				-wt*log(s*cens*gamma((t+1)/2)/gamma(t/2)*
 					exp(-p[np]/2-((t+1)/2)*
 					log(1+(y[,1]-m)^2/t))
@@ -394,8 +481,9 @@ ret <- switch(dist,
 				m <- mu1(p)
 				s <- mixt(p)
 				t <- exp(p[np]/2)
-				pc <- pcauchy(y[,1]-delta/2,m,t)
-				-wt*log(s*cens*(pcauchy(y[,1]+delta/2,m,t)-pc)
+				pc <- pcauchy(y[,1],m,t)
+				-wt*log(s*cens*(pcauchy(y[,1]+delta/2,m,t)-
+					pcauchy(y[,1]-delta/2,m,t))
 					+(1-cens)*((1+s*(rc*pc-lc))*b
 					+s*(r+pc*(l-r))))}
 			const <- 0}
@@ -404,7 +492,7 @@ ret <- switch(dist,
 				m <- mu1(p)
 				s <- mixt(p)
 				t <- exp(p[np]/2)
-				pc <- pcauchy(y[,1]-delta/2,m,t)
+				pc <- pcauchy(y[,1],m,t)
 				-wt*log(s*cens/(t*(1+(y[,1]-m)^2/t^2))
 					+(1-cens)*((1+s*(rc*pc-lc))*b
 					+s*(r+pc*(l-r))))}
@@ -415,9 +503,10 @@ ret <- switch(dist,
 				m <- mu1(p)
 				s <- mixt(p)
 				t <- exp(p[np])
-				pl <- plaplace((y[,1]-delta/2-m)/t)
+				pl <- plaplace((y[,1]-m)/t)
 				-wt*log(s*cens*(plaplace((y[,1]+delta/2-m)/t)
-					-pl)+(1-cens)*((1+s*(rc*pl-lc))*b
+					-plaplace((y[,1]-delta/2-m)/t))+
+					(1-cens)*((1+s*(rc*pl-lc))*b
 					+s*(r+pl*(l-r))))}
 			const <- 0}
 		else {
@@ -425,7 +514,7 @@ ret <- switch(dist,
 				m <- mu1(p)
 				s <- mixt(p)
 				t <- exp(p[np])
-				pl <- plaplace((y[,1]-delta/2-m)/t)
+				pl <- plaplace((y[,1]-m)/t)
 				-wt*log(s*cens*exp(-abs(y[,1]-m)/t-p[np])+
 					(1-cens)*((1+s*(rc*pl-lc))*b
 					+s*(r+pl*(l-r))))}
@@ -436,9 +525,10 @@ ret <- switch(dist,
 				m <- mu1(p)
 				s <- mixt(p)
 				t <- exp(p[np])
-				pl <- plevy(y[,1]-delta/2,m,t)
+				pl <- plevy(y[,1],m,t)
 				-wt*log(s*cens*(plevy(y[,1]+delta/2,m,t)
-					-pl)+(1-cens)*((1+s*(rc*pl-lc))*b
+					-plevy(y[,1]-delta/2,m,t))+
+					(1-cens)*((1+s*(rc*pl-lc))*b
 					+s*(r+pl*(l-r))))}
 			const <- 0}
 		else {
@@ -446,7 +536,7 @@ ret <- switch(dist,
 				m <- mu1(p)
 				s <- mixt(p)
 				t <- exp(p[np])
-				pl <- plevy(y[,1]-delta/2,m,t)
+				pl <- plevy(y[,1],m,t)
 				-wt*log(s*cens*sqrt(t/(2*pi))*log(y[,1]-m)^-1.5
 					*exp(-t/(2*(y[,1]-m)))+(1-cens)*
 					((1+s*(rc*pl-lc))*b+s*(r+pl*(l-r))))}
@@ -457,8 +547,9 @@ ret <- switch(dist,
 				s <- mixt(p)
 				u <- exp(p[np])
 				t <- 1/(mu1(p)*u)
-				pp <- (1+(y[,1]-delta/2)*t)^-u
-				-wt*log(s*cens*(pp-(1+(y[,1]+delta/2)*t)^-u)
+				pp <- 1-(1+y[,1]*t)^-u
+				-wt*log(s*cens*((1+(y[,1]-delta/2)*t)^-u-
+					(1+(y[,1]+delta/2)*t)^-u)
 					+(1-cens)*((1+s*(rc*pp-lc))*b
 					+s*(r+pp*(l-r))))}
 			const <- 0}
@@ -467,7 +558,7 @@ ret <- switch(dist,
 				s <- mixt(p)
 				u <- exp(p[np])
 				t <- 1/(mu1(p)*u)
-				pp <- 1-(1+(y[,1]-delta/2)*t)^-u
+				pp <- 1-(1+y[,1]*t)^-u
 				-wt*log(s*cens*u*t*(1+y[,1]*t)^(-(u+1))+
 					(1-cens)*
 					((1+s*(rc*pp-lc))*b+s*(r+pp*(l-r))))}
@@ -477,8 +568,9 @@ ret <- switch(dist,
 			fcn <- function(p) {
 				m <- mu1(p)
 				s <- mixt(p)
-				u <- exp(-(y[,1]-delta/2)/m)
-				-wt*log(s*cens*(-exp(-(y[,1]+delta/2)/m)+u)
+				u <- exp(-y[,1]/m)
+				-wt*log(s*cens*(-exp(-(y[,1]+delta/2)/m)+
+					exp(-(y[,1]-delta/2)/m))
 					+(1-cens)*((1+s*(rc*(1-u)-lc))*b
 					+s*(r+(1-u)*(l-r))))}
 			const <- 0}
@@ -486,7 +578,7 @@ ret <- switch(dist,
 			fcn <- function(p) {
 				m <- mu1(p)
 				s <- mixt(p)
-				u <- exp(-(y[,1]-delta/2)/m)
+				u <- exp(-y[,1]/m)
 				-wt*log(s*cens*exp(-y[,1]/m)/m
 					+(1-cens)*((1+s*(rc*(1-u)-lc))*b
 					+s*(r+(1-u)*(l-r))))}
@@ -498,8 +590,9 @@ ret <- switch(dist,
 				s <- mixt(p)
 				t <- exp(p[np])
 				u <- m/t
-				pg <- pgamma(y[,1]-delta/2,t,u)
-				-wt*log(s*cens*(pgamma(y[,1]+delta/2,t,u)-pg)
+				pg <- pgamma(y[,1],t,u)
+				-wt*log(s*cens*(pgamma(y[,1]+delta/2,t,u)-
+					pgamma(y[,1]-delta/2,t,u))
 					+(1-cens)*((1+s*(rc*pg-lc))*b
 					+s*(r+pg*(l-r))))}
 			const <- 0}
@@ -509,7 +602,7 @@ ret <- switch(dist,
 				s <- mixt(p)
 				t <- exp(p[np])
 				u <- m/t
-				pg <- pgamma(y[,1]-delta/2,t,u)
+				pg <- pgamma(y[,1],t,u)
 				-wt*log(s*cens*y[,1]^(t-1)*exp(-y[,1]/u)/
 					(u^t*gamma(t))
 					+(1-cens)*((1+s*(rc*pg-lc))*b
@@ -521,8 +614,9 @@ ret <- switch(dist,
 				m <- mu1(p)
 				s <- mixt(p)
 				t <- exp(p[np])
-				pw <- pweibull(y[,1]-delta/2,t,m)
-				-wt*log(s*cens*(pweibull(y[,1]+delta/2,t,m)-pw)
+				pw <- pweibull(y[,1],t,m)
+				-wt*log(s*cens*(pweibull(y[,1]+delta/2,t,m)-
+					pweibull(y[,1]-delta/2,t,m))
 					+(1-cens)*((1+s*(rc*pw-lc))*b
 					+s*(r+pw*(l-r))))}
 			const <- 0}
@@ -543,9 +637,11 @@ ret <- switch(dist,
 				m <- mu1(p)
 				s <- mixt(p)
 				t <- exp(p[np])
-				pw <- pweibull(exp(y[,1])-delta/2,t,m)
-				-wt*log(s*cens*(pweibull(exp(y[,1])+delta/2,
-					t,m)-pw)+(1-cens)*((1+s*(rc*pw-lc))*b
+				ey <- exp(y[,1])
+				pw <- pweibull(ey,t,m)
+				-wt*log(s*cens*(pweibull(ey+ey*delta/2,
+					t,m)-pweibull(ey-ey*delta/2,t,m))+
+					(1-cens)*((1+s*(rc*pw-lc))*b
 					+s*(r+pw*(l-r))))}
 			const <- 0}
 		else {
@@ -562,45 +658,35 @@ ret <- switch(dist,
 	own={const <- 0})
 fn <- function(p) sum(fcn(p))
 if(fscale==1)fscale <- fn(p)
-if(is.na(fscale))
-	stop("Non-numerical function value: probably invalid initial values")
+if(is.na(fn(p)))stop("Likelihood returns NAs: probably invalid initial values")
 z0 <- nlm(fn, p=p, hessian=T, print.level=print.level, typsiz=typsiz,
 	ndigit=ndigit, gradtol=gradtol, stepmax=stepmax, steptol=steptol,
 	iterlim=iterlim, fscale=fscale)
 z0$minimum <- z0$minimum+sum(const)
-if(!is.language(lin1))cname <- paste("p",1:npl,sep="")
-else {
-     cname <- colnames(dm1)
-     if(is.function(mu)&&length(cname)<npl)
-	cname <- c(cname,paste("p",(length(cname)+1):npl,sep=""))}
-if(!is.language(lin2))mname <- paste("p",1:npm,sep="")
-else {
-     mname <- colnames(dm2)
-     if(is.function(mix)&&length(mname)<npm)
-	mname <- c(mname,paste("p",(length(mname)+1):npm,sep=""))}
-fitted.values <- if(dist=="binomial"||dist=="beta binomial"||
-		dist=="double binomial"||dist=="mult binomial")
+fitted.values <- if(distribution=="binomial"||distribution=="beta binomial"||
+		distribution=="double binomial"||distribution=="mult binomial")
 		as.vector((y[,1]+y[,2])*mu1(z0$estimate))
 	else as.vector(mu1(z0$estimate))
-residuals <- if(dist!="Poisson"&&dist!="negative binomial"&&dist!="Consul"
-	&&dist!="double Poisson"&&dist!="mult Poisson"&&dist!="gamma count")
+residuals <- if(distribution!="Poisson"&&distribution!="negative binomial"&&
+	distribution!="Consul"&&distribution!="double Poisson"&&
+	distribution!="mult Poisson"&&distribution!="gamma count")
 		y[,1]-fitted.values
 	else y-fitted.values
-if(np==1){
-	cov <- 1/z0$hessian
-	se <- sqrt(cov)}
+if(np==1)cov <- 1/z0$hessian
 else {
 	a <- qr(z0$hessian)
 	if(a$rank==np)cov <- solve(z0$hessian)
-	else cov <- matrix(NA,ncol=np,nrow=np)
-	se <- sqrt(diag(cov))}
+	else cov <- matrix(NA,ncol=np,nrow=np)}
+se <- sqrt(diag(cov))
 like.comp <- as.vector(fcn(z0$estimate)+const)
-if(is.function(mu))mu1 <- mu
-if(is.function(mix))mixt <- mix
+if(!is.null(mu2))mu1 <- mu2
+if(!is.null(mixt2))mixt <- mixt2
+if(!is.null(lin1a))lin1 <- lin1a
+if(!is.null(lin2a))lin2 <- lin2a
 z1 <- list(
 	call=call,
 	delta=delta,
-	dist=dist,
+	distribution=distribution,
 	likefn=fcn,
 	mu=mu1,
 	mix=mixt,
@@ -614,9 +700,6 @@ z1 <- list(
 	aic=z0$minimum+np,
 	df=sum(wt)-np,
 	coefficients=z0$estimate,
-	cname=cname,
-	sname="p1",
-	mname=mname,
 	npl=npl,
 	npm=npm,
 	nps=as.numeric(sht),
