@@ -18,21 +18,22 @@
 #
 #  SYNOPSIS
 #
-#     gnlr(y, dist="normal", pmu=NULL, pshape=NULL, mu=NULL,
-#	shape=NULL, linear=NULL, exact=F, wt=1, delta=1, shfn=F,
-#	print.level=0, typsiz=abs(p), ndigit=10, gradtol=0.00001,
-#	stepmax=10*sqrt(p%*%p), steptol=0.00001, iterlim=100, fscale=1)
+#     gnlr(y, distribution="normal", mu=NULL, shape=NULL, linear=NULL,
+#	pmu=NULL, pshape=NULL, exact=F, wt=1, delta=1, shfn=F,
+#	envir=sys.frame(sys.parent()), print.level=0, typsiz=abs(p),
+#	ndigit=10, gradtol=0.00001, stepmax=10*sqrt(p%*%p), steptol=0.00001,
+#	iterlim=100, fscale=1)
 #
 #  DESCRIPTION
 #
 #    A function to fit nonlinear regression models with a variety of
 # one and two parameter distributions.
 
-gnlr <- function(y, dist="normal", pmu=NULL, pshape=NULL, mu=NULL,
+gnlr <- function(y, distribution="normal", pmu=NULL, pshape=NULL, mu=NULL,
 	shape=NULL, linear=NULL, exact=F, wt=1, delta=1, shfn=F,
-	print.level=0, typsiz=abs(p), ndigit=10, gradtol=0.00001,
-	stepmax=10*sqrt(p%*%p), steptol=0.00001, iterlim=100,
-	fscale=1){
+	envir=sys.frame(sys.parent()), print.level=0, typsiz=abs(p),
+	ndigit=10, gradtol=0.00001, stepmax=10*sqrt(p%*%p), steptol=0.00001,
+	iterlim=100, fscale=1){
 
 pinvgauss <- function(y,m,s){
 	t <- y/m
@@ -53,26 +54,43 @@ plevy <- function(y, m, s)
 		pts=as.integer(5),
 		max=as.integer(16),
 		err=integer(1),
-		res=double(n))$res
+		res=double(n),
+		DUP=F)$res
 
 call <- sys.call()
-if(!missing(dist)&&!is.function(dist)){
-	dist <- match.arg(dist,c("binomial","beta binomial","double binomial",
-	"mult binomial","Poisson","negative binomial","double Poisson",
-	"mult Poisson","gamma count","Consul","logarithmic","geometric",
-	"normal","inverse Gauss","logistic","exponential","gamma","Weibull",
-	"extreme value","Pareto","Cauchy","Laplace","Levy"))}
-if(!missing(pmu))npl <- length(pmu)
+if(!missing(distribution)&&!is.function(distribution)){
+	distribution <- match.arg(distribution,c("binomial","beta binomial",
+	"double binomial","mult binomial","Poisson","negative binomial",
+	"double Poisson","mult Poisson","gamma count","Consul","logarithmic",
+	"geometric","normal","inverse Gauss","logistic","exponential","gamma",
+	"Weibull","extreme value","Pareto","Cauchy","Laplace","Levy"))}
+shp <- distribution!="binomial"&&distribution!="Poisson"&&
+	distribution!="exponential"&&distribution!="geometric"&&
+	distribution!="logarithmic"
+if(!is.null(pmu))npl <- length(pmu)
 else npl <- 0
-if(!missing(pshape))nps <- length(pshape)
+if(!is.null(pshape))nps <- length(pshape)
 else nps <- 0
 np <- npl+nps
-p <- c(pmu,pshape)
 if(np<1)stop("At least one parameter must be estimated")
-if(is.function(dist)){
-	fcn <- dist
-	dist <- "own"}
-if(dist=="binomial"||dist=="double binomial"||dist=="beta binomial"||dist=="mult binomial"){
+if(is.function(distribution)){
+	fcn <- distribution
+	distribution <- "own"}
+if(inherits(y,"repeated")){
+	if(inherits(envir,"repeated")&&(length(y$response$nobs)!=length(envir$response$nobs)||any(y$response$nobs!=envir$response$nobs)))stop("y and envir objects are incompatible")
+	if(!is.null(y$response$wt)&&!is.na(y$response$wt))wt <- y$response$wt
+	if(!is.null(y$response$delta))delta <- y$response$delta
+	if(!is.null(y$response$censor)&&any(y$response$censor!=1))y <- cbind(y$response$y,y$response$censor)
+	else if(!is.null(y$response$n))y <- cbind(y$response$y,y$response$n-y$response$y)
+	else y <- y$response$y}
+else if(inherits(y,"response")){
+	if(!is.null(y$wt)&&!is.na(y$wt))wt <- y$wt
+	if(!is.null(y$delta))delta <- y$delta
+	if(!is.null(y$censor)&&any(y$censor!=1))y <- cbind(y$y,y$censor)
+	else if(!is.null(y$n))y <- cbind(y$y,y$n-y$y)
+	else y <- y$y}
+if(distribution=="binomial"||distribution=="double binomial"||
+	distribution=="beta binomial"||distribution=="mult binomial"){
 	if(length(dim(y))!=2||ncol(y)!=2)
 		stop(paste("Two column matrix required for response: successes and failures"))
 	if(any(y<0))stop("All response values must be positive")
@@ -80,22 +98,24 @@ if(dist=="binomial"||dist=="double binomial"||dist=="beta binomial"||dist=="mult
 	nn <- y[,1]+y[,2]
 	censor <- F}
 else {
-	if(inherits(y,"response")){
-		if(is.null(y$censor))y <- y$y
-		else y <- cbind(y$y,y$censor)}
 	censor <- length(dim(y))==2&&ncol(y)==2
+	if(censor&&all(y[,2]==1)){
+		y <- y[,1]
+		censor <- F}
 	if(!censor){
-		if(!is.vector(y,mode="double"))stop("y must be a vector")
+		if(!is.vector(y,mode="numeric"))stop("y must be a vector")
 		n <- length(y)
-		if(dist=="double Poisson"||dist=="mult Poisson")
+		if(distribution=="double Poisson"||distribution=="mult Poisson")
 			my <- 3*max(y)}}
-if((dist=="inverse Gauss"||dist=="exponential"||dist=="gamma"||
-	dist=="Weibull"||dist=="extreme value")&&((censor&&any(y[,1]<=0))||
+if((distribution=="inverse Gauss"||distribution=="exponential"||
+	distribution=="gamma"||distribution=="Weibull"||
+	distribution=="extreme value")&&((censor&&any(y[,1]<=0))||
 	(!censor&&any(y<=0))))stop("All response values must be > 0")
-if((dist=="Poisson"||dist=="negative binomial"||dist=="gamma count"||
-	dist=="double Poisson"||dist=="mult Poisson")&&(any(y<0)))
+if((distribution=="Poisson"||distribution=="negative binomial"||
+	distribution=="gamma count"||distribution=="double Poisson"||
+	distribution=="mult Poisson")&&(any(y<0)))
 	stop("All response values must be >= 0")
-if(dist=="logarithmic"&&any(y[wt>0]<1))
+if(distribution=="logarithmic"&&any(y[wt>0]<1))
 	stop("All response values must be integers > 0")
 if(censor){
 	n <- nrow(y)
@@ -120,86 +140,138 @@ else if(length(delta)!=n)stop("delta must be the same length as the other variab
 lin1 <- lin2 <- NULL
 if(is.list(linear)){
 	lin1 <- linear[[1]]
-	lin2 <- linear[[2]]
-	lsh <- is.language(lin2)}
-else {
-	lin1 <- linear
-	lin2 <- NULL
-	lsh <- FALSE}
-if(is.language(mu))lin1 <- mu
-if(is.language(shape)){
-	lin2 <- shape
-	lsh <- is.language(lin2)}
-nlp <- npl
-if(is.language(lin1)){
-	mt <- terms(lin1)
-	if(is.numeric(mt[[2]])){
-		dm1 <- matrix(1)
-		colnames(dm1) <- "(Intercept)"
-		npt1 <- 1
-		if(!is.function(mu)){
-			mu1 <- function(p) p[1]*rep(1,n)
-			nlp <- 1}
-		else mu1 <- function(p) mu(p, p[1]*rep(1,n))}
+	lin2 <- linear[[2]]}
+else lin1 <- linear
+if(inherits(mu,"formula"))lin1 <- mu
+if(inherits(shape,"formula"))lin2 <- shape
+lin1a <- lin2a <- mu2 <- sh2 <- name <- NULL
+if(inherits(envir,"repeated")||inherits(envir,"tccov")){
+	type <- if(inherits(envir,"repeated"))"repeated"
+		else "tccov"
+	if(inherits(lin1,"formula")){
+		lin1a <- finterp(lin1)
+		class(lin1) <- c(class(lin1),type)}
+	if(inherits(lin2,"formula")){
+		lin2a <- finterp(lin2)
+		class(lin2) <- c(class(lin2),type)}
+	name <- paste(deparse(substitute(envir)))
+	if(is.function(mu)){
+		mu2 <- mu
+		attributes(mu2) <- attributes(fnenvir(mu))
+		class(mu) <- type
+		mu <- fnenvir(mu,envir=envir,name=name)}
+	if(is.function(shape)){
+		sh2 <- shape
+		attributes(sh2) <- attributes(fnenvir(shape))
+		class(shape) <- type
+		shape <- fnenvir(shape,envir=envir,name=name)}}
+if(inherits(lin1,"formula")){
+	mu1 <- finterp(lin1,envir=envir,name=name)
+	npt1 <- length(attr(mu1,"parameters"))
+	if(is.matrix(attr(mu1,"model"))){
+		if(all(dim(attr(mu1,"model"))==1)){
+			if(is.function(mu)){
+				lin1 <- mu1
+				mu1 <- function(p) mu(p,p[1]*rep(1,n))}
+			else {
+				tmp <- attributes(mu1)
+				mu1 <- function(p) p[1]*rep(1,n)
+				attributes(mu1) <- tmp}}
+		else {
+			if(nrow(attr(mu1,"model"))!=n)stop("mu model matrix does not match number of response observations")
+			if(is.function(mu)){
+				dm1 <- attr(mu1,"model")
+				lin1 <- mu1
+				mu1 <- function(p) mu(p,dm1%*%p[1:npt1])}}}
 	else {
-		mf <- model.frame(mt,sys.frame(sys.parent()),na.action=na.fail)
-		dm1 <- model.matrix(mt, mf)
-		npt1 <- ncol(dm1)
-		if(!is.function(mu)){
-			mu1 <- function(p) dm1%*%p[1:npt1]
-			nlp <- npt1}
-		else mu1 <- function(p) mu(p, dm1%*%p[1:npt1])}
-	if(npl<npt1)stop("Not enough initial estimates for mu")}
+		if(npl!=npt1){
+			cat("\nParameters are ")
+			cat(attr(mu1,"parameters"),"\n")
+			stop(paste("pmu should have",npt1,"estimates"))}
+		if(is.list(pmu)){
+			if(!is.null(names(pmu))){
+				o <- match(attr(mu1,"parameters"),names(pmu))
+				pmu <- unlist(pmu)[o]
+				if(sum(!is.na(o))!=length(pmu))stop("invalid estimates for mu - probably wrong names")}
+			else pmu <- unlist(pmu)}}}
 else if(!is.function(mu)){
 	mu1 <- function(p) p[1]*rep(1,n)
-	nlp <- 1}
+	npt1 <- 1}
 else {
 	mu1 <- mu
-	if(length(mu1(p))==1)mu1 <- function(p) mu(p)*rep(1,n)}
-if(nlp!=npl)
-	stop("Number of initial estimates for mu does not correspond to model")
+	if(length(mu1(pmu))==1)mu1 <- function(p) mu(p)*rep(1,n)}
+if(is.null(attributes(mu1))){
+	attributes(mu1) <- if(is.function(mu)){
+		if(!inherits(mu,"formulafn"))attributes(fnenvir(mu))
+		else attributes(mu)}
+		else attributes(fnenvir(mu1))}
+nlp <- if(is.function(mu)){
+		if(is.null(lin1))length(attr(mu1,"parameters"))
+		else length(attr(mu1,"parameters"))-1+npt1}
+       else npt1
+if(nlp!=npl)stop(paste("pmu should have",nlp,"initial estimates"))
+if(any(is.na(mu1(pmu))))stop("The location model returns NAs: probably invalid initial values")
 npl1 <- npl+1
-nlp <- nps
-if(lsh){
-	mt <- terms(lin2)
-	if(is.numeric(mt[[2]])){
-		dm2 <- matrix(1)
-		colnames(dm2) <- "(Intercept)"
-		npt2 <- 1
-		if(!is.function(shape)){
-			sh1 <- function(p) p[npl1]*rep(1,n)
-			nlp <- 1}
-		else sh1 <- function(p) shape(p[npl1:np], p[npl1]*rep(1,n))}
-	else {
-		mf <- model.frame(mt,sys.frame(sys.parent()),na.action=na.fail)
-		dm2 <- model.matrix(mt, mf)
-		npt2 <- ncol(dm2)
-		if(!is.function(shape)){
-			sh1 <- function(p) dm2%*%p[npl1:(npl1+npt2-1)]
-			nlp <- npt2}
+if(inherits(lin2,"formula")){
+	sh1 <- finterp(lin2,envir=envir,start=npl1,name=name)
+	npt2 <- length(attr(sh1,"parameters"))
+	if(is.matrix(attr(sh1,"model"))){
+		if(all(dim(attr(sh1,"model"))==1)){
+			if(is.function(shape)){
+				lin2 <- sh1
+				sh1 <- if(shfn)function(p) shape(p[npl1:np],p[npl1]*rep(1,n), mu1(p))
+					else function(p) shape(p[npl1:np],p[npl1])}
+			else {
+				tmp <- attributes(sh1)
+				sh1 <- function(p) p[npl1]*rep(1,n)
+				sh2 <- fnenvir(function(p) p[1]*rep(1,n))
+				attributes(sh1) <- tmp}}
 		else {
-			if(shfn)sh1 <- function(p) shape(p[npl1:np], dm2%*%p[npl1:(npl1+npt2-1)], mu1(p))
-			else sh1 <- function(p) shape(p[npl1:np], dm2%*%p[npl1:(npl1+npt2-1)])}}
-	if(nps<npt2)stop("Not enough initial estimates for shape")}
-else if(!is.function(shape)&&dist!="binomial"&&dist!="Poisson"&&
-	dist!="exponential"&&dist!="geometric"&&dist!="logarithmic"){
+			if(nrow(attr(sh1,"model"))!=n)stop("shape model matrix does not match number of response observations")
+			if(is.function(shape)){
+				dm2 <- attr(sh1,"model")
+				lin2 <- sh1
+				sh1 <- if(shfn)function(p) shape(p[npl1:np], dm2%*%p[npl1:(npl1+npt2-1)], mu1(p))
+					else function(p) shape(p[npl1:np],dm2%*%p[npl1:(npl1+npt2-1)])}}}
+	else {
+		if(nps!=npt2){
+			cat("\nParameters are ")
+			cat(attr(sh1,"parameters"),"\n")
+			stop(paste("pshape should have",npt2,"estimates"))}
+		if(is.list(pshape)){
+			if(!is.null(names(pshape))){
+				o <- match(attr(sh1,"parameters"),names(pshape))
+				pshape <- unlist(pshape)[o]
+				if(sum(!is.na(o))!=length(pshape))stop("invalid estimates for shape - probably wrong names")}
+			else pshape <- unlist(pshape)}}}
+else if(!is.function(shape)&&shp){
 	sh1 <- function(p) p[npl1]*rep(1,n)
-	nlp <- 1}
+	sh2 <- fnenvir(function(p) p[1]*rep(1,n))
+	npt2 <- 1}
 else {
-	if(shfn)sh1 <- function(p) shape(p[npl1:np], mu1(p))
-	else sh1 <- function(p) shape(p[npl1:np])}
-if(nlp!=nps)
-	stop("Number of initial estimates for shape does not correspond to model")
-if(!is.numeric(mu1(pmu)))
-	stop("The location function must return numerical values")
-if(dist=="Levy"&&((!censor&&any(y<=mu1(p)))||(censor&&any(y[,1]<=mu1(p)))))
+	sh1 <- if(shfn)function(p) shape(p[npl1:np], mu1(p))
+		else function(p) shape(p[npl1:np])}
+if(shp){
+	if(is.null(attributes(sh1))){
+		attributes(sh1) <- if(is.function(shape)){
+			if(!inherits(shape,"formulafn"))attributes(fnenvir(shape))
+			else attributes(shape)}
+			else attributes(fnenvir(sh1))}
+	nlp <- if(is.function(shape)){
+			if(is.null(lin2))length(attr(sh1,"parameters"))-shfn
+			else length(attr(sh1,"parameters"))-1+npt2-shfn}
+		else npt2
+	if(nlp!=nps)stop(paste("pshape should have",nlp,"initial estimates"))}
+p <- c(pmu,pshape)
+if(distribution=="Levy"&&((!censor&&any(y<=mu1(p)))||(censor&&any(y[,1]<=mu1(p)))))
 	stop("location parameter must be strictly less than corresponding observation")
-if(dist=="Pareto"&&exp(sh1(p))<=1)stop("shape parameters must be > 0")
-if(dist!="binomial"&&dist!="Poisson"&&dist!="exponential"&&
-	dist!="geometric"&&dist!="logarithmic"&&!is.numeric(sh1(p)))
-	stop("The shape function must return numerical values")
+if(distribution=="Pareto"&&exp(sh1(p))<=1)stop("shape parameters must be > 0")
+if(distribution!="binomial"&&distribution!="Poisson"&&
+	distribution!="exponential"&&distribution!="geometric"&&
+	distribution!="logarithmic"&&any(is.na(sh1(p))))
+	stop("The shape model returns NAs: probably invalid initial values")
 if (!censor){
-	ret <- switch(dist,
+	ret <- switch(distribution,
 	binomial={
 		fcn <- function(p) {
 			m <- mu1(p)
@@ -217,13 +289,13 @@ if (!censor){
 		fcn <- function(p) {
 			-.C("ddb",as.integer(y[,1]),as.integer(nn),
 				as.double(mu1(p)),as.double(exp(sh1(p))),
-				as.integer(n),as.double(wt),res=double(n))$res}
+				as.integer(n),as.double(wt),res=double(n),DUP=F)$res}
 		const <- 0},
 	"mult binomial"={
 		fcn <- function(p) {
 			-.C("dmb",as.integer(y[,1]),as.integer(nn),
 				as.double(mu1(p)),as.double(exp(sh1(p))),
-				as.integer(n),as.double(wt),res=double(n))$res}
+				as.integer(n),as.double(wt),res=double(n),DUP=F)$res}
 		const <- 0},
 	Poisson={
 		fcn <- function(p) {
@@ -243,14 +315,14 @@ if (!censor){
 			-.C("ddp",as.integer(y),as.integer(my),
 				as.double(mu1(p)),as.double(exp(sh1(p))),
 				as.integer(length(y)),as.double(wt),
-				res=double(length(y)))$res}
+				res=double(length(y)),DUP=F)$res}
 		const <- 0},
 	"mult Poisson"={
 		fcn <- function(p) {
 			-.C("dmp",as.integer(y),as.integer(my),
 				as.double(mu1(p)),as.double(exp(sh1(p))),
 				as.integer(length(y)),as.double(wt),
-				res=double(length(y)))$res}
+				res=double(length(y)),DUP=F)$res}
 		const <- 0},
 	"gamma count"={
 		fcn <- function(p) {
@@ -438,7 +510,7 @@ if (!censor){
 			const <- -wt*log(delta)}},
 	own={ const <- 0})}
 else {
-	ret <- switch(dist,
+	ret <- switch(distribution,
 	Poisson={
 		fcn <- function(p) {
 			m <- mu1(p)
@@ -465,9 +537,9 @@ else {
 			fcn <- function(p) {
 				m <- mu1(p)
 				s <- exp(sh1(p)/2)
-				pn <- pnorm(y[,1]-delta/2,m,s)
-				-wt*(cc*log(pnorm(y[,1]+delta/2,m,s)-pn)
-					+log(lc-rc*pn))}
+				-wt*(cc*log(pnorm(y[,1]+delta/2,m,s)-
+					pnorm(y[,1]-delta/2,m,s))
+					+log(lc-rc*pnorm(y[,1],m,s)))}
 			const <- 0}
 		else {
 			fcn <- function(p) {
@@ -475,16 +547,16 @@ else {
 				t <- sh1(p)
 				s <- exp(t)
 				-wt*(cc*(-(t+(y[,1]-m)^2/s)/2)+log(lc-rc
-					*pnorm(y[,1]-delta/2,m,sqrt(s))))}
+					*pnorm(y[,1],m,sqrt(s))))}
 			const <- wt*cc*(log(2*pi)/2-log(delta))}},
         "inverse Gauss"={
 		if(exact){
 			fcn <- function(p) {
 				m <- mu1(p)
 				s <- exp(sh1(p))
-				pit <- pinvgauss(y[,1]-delta/2,m,s)
-				-wt*(cc*log(pinvgauss(y[,1]+delta/2,m,s)-pit)
-					+log(lc-rc*pit))}
+				-wt*(cc*log(pinvgauss(y[,1]+delta/2,m,s)-
+					pinvgauss(y[,1]-delta/2,m,s))
+					+log(lc-rc*pinvgauss(y[,1],m,s)))}
 			const <- 0}
 		else {
 			fcn <- function(p) {
@@ -492,16 +564,16 @@ else {
 				t <- sh1(p)
 				s <- exp(t)
 				-wt*(cc*(-(t+(y[,1]-m)^2/(y[,1]*s*m^2))/2)+
-					log(lc-rc*pinvgauss(y[,1]-delta/2,m,s)))}
+					log(lc-rc*pinvgauss(y[,1],m,s)))}
 			const <- wt*cc*(log(2*pi*y[,1]^3)/2-log(delta))}},
 	logistic={
 		if(exact){
 			fcn <- function(p) {
 				m <- mu1(p)
 				s <- exp(sh1(p))*sqrt(3)/pi
-				pl <- plogis(y[,1]-delta/2,m,s)
-				-wt*(cc*log(plogis(y[,1]+delta/2,m,s)-pl)
-					+log(lc-rc*pl))}
+				-wt*(cc*log(plogis(y[,1]+delta/2,m,s)-
+					plogis(y[,1]-delta/2,m,s))
+					+log(lc-rc*plogis(y[,1],m,s)))}
 			const <- 0}
 		else {
 			fcn <- function(p) {
@@ -509,33 +581,32 @@ else {
 				s <- exp(sh1(p))*sqrt(3)/pi
 				y1 <- (y[,1]-m)/s
 				-wt*(cc*(-y1-log(s)-2*log(1+exp(-y1)))
-					+log(lc-rc*plogis(y[,1]-delta/2,m,s)))}
+					+log(lc-rc*plogis(y[,1],m,s)))}
 			const <- -wt*cc*log(delta)}},
 	Cauchy={
 		if(exact){
 			fcn <- function(p) {
 				m <- mu1(p)
 				s <- exp(sh1(p)/2)
-				pc <- pcauchy(y[,1]-delta/2,m,s)
-				-wt*(cc*log(pcauchy(y[,1]+delta/2,m,s)-pc)
-					+log(lc-rc*pc))}
+				-wt*(cc*log(pcauchy(y[,1]+delta/2,m,s)-
+					pcauchy(y[,1]-delta/2,m,s))
+					+log(lc-rc*pcauchy(y[,1],m,s)))}
 			const <- 0}
 		else {
 			fcn <- function(p) {
 				m <- mu1(p)
 				s <- exp(sh1(p)/2)
 				-wt*(-cc*log(s*(1+((y[,1]-m)/s)^2))
-					+log(lc-rc*pcauchy(y[,1]-delta/2
-					,m,s)))}
+					+log(lc-rc*pcauchy(y[,1],m,s)))}
 			const <- -wt*cc*log(delta/pi)}},
         Laplace={
 		if(exact){
 			fcn <- function(p) {
 				m <- mu1(p)
 				s <- exp(sh1(p))
-				pl <- plaplace(y[,1]-delta/2,m,s)
-				-wt*(cc*log(plaplace(y[,1]+delta/2,m,s)-pl)
-					+log(lc-rc*pl))}
+				-wt*(cc*log(plaplace(y[,1]+delta/2,m,s)-
+					plaplace(y[,1]-delta/2,m,s))
+					+log(lc-rc*plaplace(y[,1],m,s)))}
 			const <- 0}
 		else {
 			fcn <- function(p) {
@@ -543,16 +614,16 @@ else {
 				t <- sh1(p)
 				s <- exp(t)
 				-wt*(cc*(-abs(y[,1]-m)/s-t)+log(lc-rc
-					*plaplace(y[,1]-delta/2,m,s)))}
+					*plaplace(y[,1],m,s)))}
 			const <- -wt*cc*log(delta/2)}},
         Levy={
 		if(exact){
 			fcn <- function(p) {
 				m <- mu1(p)
 				s <- exp(sh1(p))
-				pl <- plevy(y[,1]-delta/2,m,s)
-				-wt*(cc*log(plevy(y[,1]+delta/2,m,s)-pl)
-					+log(lc-rc*pl))}
+				-wt*(cc*log(plevy(y[,1]+delta/2,m,s)-
+					plevy(y[,1]-delta/2,m,s))
+					+log(lc-rc*plevy(y[,1],m,s)))}
 			const <- 0}
 		else {
 			fcn <- function(p) {
@@ -561,38 +632,37 @@ else {
 				s <- exp(t)
 				-wt*(cc*(0.5*log(s/(2*pi))-1.5*log(y[,1]-m)-
 					s/(2*(y[,1]-m)))+log(lc-rc
-					*plevy(y[,1]-delta/2,m,s)))}
+					*plevy(y[,1],m,s)))}
 			const <- -wt*cc*log(delta/2)}},
         Pareto={
 		if(exact){
 			fcn <- function(p) {
 				s <- exp(sh1(p))
 				t <- 1/(mu1(p)*(s-1))
-				pp <- (1+(y[,1]-delta/2)*t)^-s
-				-wt*(cc*log(pp-(1+(y[,1]+delta/2)*t)^-s)
-					+log(lc-rc*pp))}
+				-wt*(cc*log((1+(y[,1]-delta/2)*t)^-s-
+					(1+(y[,1]+delta/2)*t)^-s)
+					+log(lc-rc*(-(1+(y[,1])*t)^-s)))}
 			const <- 0}
 		else {
 			fcn <- function(p) {
 				s <- exp(sh1(p))
 				t <- 1/(mu1(p)*(s-1))
 				-wt*(cc*(log(s*t)-(s+1)*log(1+y[,1]*t))
-					+log(lc-rc*(1-(1+(y[,1]-delta/2)*t)^
-					-s)))}
+					+log(lc-rc*(1-(1+y[,1]*t)^-s)))}
 			const <- -wt*cc*log(delta)}},
 	exponential={
 		if(exact){
 			fcn <- function(p) {
 				m <- mu1(p)
-				pe <- exp(-(y[,1]-delta/2)/m)
 				-wt*(cc*log(-exp(-(y[,1]+delta/2)/m)
-					+pe)+log(lc-rc*(1-pe)))}
+					+exp(-(y[,1]-delta/2)/m))+
+					log(lc-rc*(1-exp(-y[,1]/m))))}
 			const <- 0}
 		else {
 			fcn <- function(p) {
 				m <- mu1(p)
 				-wt*(cc*(-log(m)-y[,1]/m)+log(lc-rc*
-					(1-exp(-(y[,1]-delta/2)/m))))}
+					(1-exp(-y[,1]/m))))}
 			const <- -wt*cc*log(delta)}},
         gamma={
 		if(exact){
@@ -600,9 +670,9 @@ else {
 				m <- mu1(p)
 				s <- exp(sh1(p))
 				u <- m/s
-				pg <- pgamma(y[,1]-delta/2,s,u)
-				-wt*(cc*log(pgamma(y[,1]+delta/2,s,u)-pg)
-					+log(lc-rc*pg))}
+				-wt*(cc*log(pgamma(y[,1]+delta/2,s,u)-
+					pgamma(y[,1]-delta/2,s,u))
+					+log(lc-rc*pgamma(y[,1],s,u)))}
 			const <- 0}
 		else {
 			fcn <- function(p) {
@@ -611,16 +681,16 @@ else {
 				s <- exp(t)
 				-wt*(cc*(s*(t-log(m)-y[,1]/m)+(s-1)*log(y[,1])
 					-lgamma(s))+log(lc-rc
-					*pgamma(y[,1]-delta/2,s,m/s)))}
+					*pgamma(y[,1],s,m/s)))}
 			const <- -wt*cc*log(delta)}},
         Weibull={
 		if(exact){
 			fcn <- function(p) {
 				m <- mu1(p)
 				s <- exp(sh1(p))
-				pw <- pweibull(y[,1]-delta/2,s,m)
-				-wt*(cc*log(pweibull(y[,1]+delta/2,s,m)-pw)
-					+log(lc-rc*pw))}
+				-wt*(cc*log(pweibull(y[,1]+delta/2,s,m)-
+					pweibull(y[,1]-delta/2,s,m))
+					+log(lc-rc*pweibull(y[,1],s,m)))}
 			const <- 0}
 		else {
 			fcn <- function(p) {
@@ -629,7 +699,7 @@ else {
 				s <- exp(t)
 				-wt*(cc*(t+(s-1)*log(y[,1])-s*log(m)
 					-(y[,1]/m)^s)+log(lc-rc*
-					pweibull(y[,1]-delta/2,s,m)))}
+					pweibull(y[,1],s,m)))}
 			const <- -wt*cc*log(delta)}},
         "extreme value"={
 		if(exact){
@@ -638,8 +708,9 @@ else {
 				s <- exp(sh1(p))
 				ey <- exp(y[,1])
 				pw <- pweibull(ey-ey*delta/2,s,m)
-				-wt*(cc*log(pweibull(ey+ey*delta/2,s,m)-pw)
-					+log(lc-rc*pw))}
+				-wt*(cc*log(pweibull(ey+ey*delta/2,s,m)-
+					pweibull(ey-ey*delta/2,s,m))
+					+log(lc-rc*pweibull(ey,s,m)))}
 			const <- 0}
 		else {
 			fcn <- function(p) {
@@ -648,48 +719,39 @@ else {
 				s <- exp(t)
 				ey <- exp(y[,1])
 				-wt*(cc*(t+s*y[,1]-s*log(m)-(ey/m)^s)+log(lc-
-					rc*pweibull(ey-ey*delta/2,s,m)))}
+					rc*pweibull(ey,s,m)))}
 			const <- -wt*cc*log(delta)}},
 	own={const <- 0})}
 fn <- function(p) sum(fcn(p))
 if(fscale==1)fscale <- fn(p)
-if(is.na(fscale))
-	stop("Non-numerical function value: probably invalid initial values")
+if(is.na(fn(p)))stop("Likelihood returns NAs: probably invalid initial values")
 z0 <- nlm(fn, p=p, hessian=T, print.level=print.level, typsiz=typsiz,
 	ndigit=ndigit, gradtol=gradtol, stepmax=stepmax, steptol=steptol,
 	iterlim=iterlim, fscale=fscale)
 z0$minimum <- z0$minimum+sum(const)
-if(!is.language(lin1))cname <- paste("p",1:npl,sep="")
-else {
-     cname <- colnames(dm1)
-     if(is.function(mu)&&length(cname)<npl)
-	cname <- c(cname,paste("p",(length(cname)+1):npl,sep=""))}
-if(!is.language(lin2))sname <- paste("p",1:nps,sep="")
-else {
-     sname <- colnames(dm2)
-     if(is.function(shape)&&length(sname)<nps)
-	sname <- c(sname,paste("p",(length(sname)+1):nps,sep=""))}
-fitted.values <- if(dist=="binomial"||dist=="beta binomial"||dist=="double binomial"||dist=="mult binomial")
+fitted.values <- if(distribution=="binomial"||distribution=="beta binomial"||
+	distribution=="double binomial"||distribution=="mult binomial")
 		as.vector((y[,1]+y[,2])*mu1(z0$estimate))
 	else as.vector(mu1(z0$estimate))
-residuals <- if(dist=="binomial"||dist=="beta binomial"||dist=="double binomial"||dist=="mult binomial"||censor)
+residuals <- if(distribution=="binomial"||distribution=="beta binomial"||
+	distribution=="double binomial"||distribution=="mult binomial"||censor)
 		y[,1]-fitted.values
 	else y-fitted.values
-if(np==1){
-	cov <- 1/z0$hessian
-	se <- sqrt(cov)}
+if(np==1)cov <- 1/z0$hessian
 else {
 	a <- qr(z0$hessian)
 	if(a$rank==np)cov <- solve(z0$hessian)
-	else cov <- matrix(NA,ncol=np,nrow=np)
-	se <- sqrt(diag(cov))}
+	else cov <- matrix(NA,ncol=np,nrow=np)}
+se <- sqrt(diag(cov))
 like.comp <- as.vector(fcn(z0$estimate)+const)
-if(is.function(mu))mu1 <- mu
-if(is.function(shape))sh1 <- shape
+if(!is.null(mu2))mu1 <- mu2
+if(!is.null(sh2))sh1 <- sh2
+if(!is.null(lin1a))lin1 <- lin1a
+if(!is.null(lin2a))lin2 <- lin2a
 z1 <- list(
 	call=call,
 	delta=delta,
-	dist=dist,
+	distribution=distribution,
 	likefn=fcn,
 	mu=mu1,
 	shape=sh1,
@@ -703,8 +765,6 @@ z1 <- list(
 	aic=z0$minimum+np,
 	df=sum(wt)-np,
 	coefficients=z0$estimate,
-	cname=cname,
-	sname=sname,
 	npl=npl,
 	npm=0,
 	nps=nps,
